package com.city.cdcmonitor.config;

import com.city.cdcmonitor.exception.CityException;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;

import javax.sql.DataSource;
import java.beans.PropertyVetoException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.Properties;
import java.util.logging.Logger;

@Slf4j
public class CityHikariDataSource implements DataSource {

    private HikariDataSource hikariDataSource;

    CityHikariDataSource() {
        super();
        init();
    }

    private void init() {
        try {
            hikariDataSource = new HikariDataSource();
            setMinPoolSize(3);// 最小连接数
            setMaxPoolSize(30);// 最大连接数
        } catch (Exception e) {
            CityException.error(e);
            throw new IllegalArgumentException("初始化数据库连接池异常" + e.getLocalizedMessage());
        }
    }

    public void setMinPoolSize(int i) {
        hikariDataSource.setMinimumIdle(i);
    }

    public int getMinPoolSize() {
        return hikariDataSource.getMinimumIdle();
    }

    public void setMaxPoolSize(int i) {
        hikariDataSource.setMaximumPoolSize(i);
    }

    public int getMaxPoolSize() {
        return hikariDataSource.getMaximumPoolSize();
    }

    private boolean isNotEmpty(String str) {
        return str != null && !"".equals(str.trim());
    }

    public void setPreferredTestQuery(String query) {
        hikariDataSource.setConnectionTestQuery(query);
    }

    public void getPreferredTestQuery() {
        hikariDataSource.getConnectionTestQuery();
    }

    private void setDriverClassByUrl(String url) {
        try {
            if (url.toLowerCase().trim().startsWith("jdbc:sqlserver")) {
                setDriverClass("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            } else if (url.toLowerCase().trim().startsWith("jdbc:postgresql")) {
                setDriverClass("org.postgresql.Driver");
                setPreferredTestQuery("select 1 ");
            } else if (url.toLowerCase().trim().startsWith("jdbc:mysql")) {
                setDriverClass("com.mysql.jdbc.Driver");
                setPreferredTestQuery("select 1 ");
            } else if (url.toLowerCase().trim().startsWith("jdbc:oracle")) {
                setDriverClass("oracle.jdbc.driver.OracleDriver");
            } else if (url.toLowerCase().trim().startsWith("jdbc:db2")) {
                setDriverClass("com.ibm.db2.jcc.DB2Driver");
                setPreferredTestQuery("select 1 from SYSIBM.sysDUMMY1 with ur");
            }
        } catch (Exception e) {
            CityException.error(e);
            throw new IllegalArgumentException("初始化数据库连接池异常" + e.getLocalizedMessage());
        }
    }

    @Override
    public PrintWriter getLogWriter() throws SQLException {
        return hikariDataSource.getLogWriter();
    }

    @Override
    public void setLogWriter(PrintWriter out) throws SQLException {
        hikariDataSource.setLogWriter(out);
    }

    @Override
    public void setLoginTimeout(int seconds) throws SQLException {
        hikariDataSource.setLoginTimeout(seconds);
    }

    @Override
    public int getLoginTimeout() throws SQLException {
        return hikariDataSource.getLoginTimeout();
    }

    @Override
    public <T> T unwrap(Class<T> iface) throws SQLException {
        return null;
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        return false;
    }

    @Override
    public Connection getConnection() throws SQLException {
        return hikariDataSource.getConnection();
    }

    @Override
    public Connection getConnection(String username, String password) throws SQLException {
        return hikariDataSource.getConnection(username, password);
    }

    public String getDriverClass() {
        return hikariDataSource.getDriverClassName();
    }

    public void setDriverClass(String driverClass) throws PropertyVetoException {
        hikariDataSource.setDriverClassName(driverClass);
    }

    public String getJdbcUrl() {
        return hikariDataSource.getJdbcUrl();
    }

    public void setJdbcUrl(String jdbcUrl) {
        hikariDataSource.setJdbcUrl(jdbcUrl);
        setDriverClassByUrl(jdbcUrl);

    }

    public Properties getProperties() {
        return hikariDataSource.getDataSourceProperties();
    }

    public void setProperties(Properties properties) {
        hikariDataSource.setDataSourceProperties(properties);
    }

    // DriverManagerDataSource "virtual properties" based on properties
    public String getUser() {
        return hikariDataSource.getUsername();
    }

    public void setUser(String user) {
        hikariDataSource.setUsername(user);
    }

    public String getPassword() {
        return hikariDataSource.getPassword();
    }

    public void setPassword(String password) {
        hikariDataSource.setPassword(password);
    }

    // WrapperConnectionPoolDataSource properties
    public long getCheckoutTimeout() {
        return hikariDataSource.getConnectionTimeout();
    }

    public void setCheckoutTimeout(long checkoutTimeout) {
        hikariDataSource.setConnectionTimeout(checkoutTimeout);
    }


    public boolean isAutoCommitOnClose() {
        return hikariDataSource.isAutoCommit();
    }

    public void setAutoCommitOnClose(boolean autoCommitOnClose) {
        hikariDataSource.setAutoCommit(autoCommitOnClose);
    }


    @Override
    public String toString() {
        return hikariDataSource.toString();

    }

    public void close() {
        hikariDataSource.close();
    }

    @Override
    public Logger getParentLogger() throws SQLFeatureNotSupportedException {
        // TODO Auto-generated method stub
        return null;
    }

}
