package com.city.cdcmonitor.controller;

import com.city.cdcmonitor.bean.InterfaceHealthBean;
import com.city.cdcmonitor.dao.jkcloud.mapper.JKCloudCommonMapper;
import com.city.cdcmonitor.dao.middb.mapper.MiddbCommonMapper;
import com.city.cdcmonitor.fegin.TokenFeign;
import com.city.cdcmonitor.service.CdcStatisticsService;
import com.city.cdcmonitor.service.CheckHealthService;
import com.city.cdcmonitor.service.SendEmailService;
import com.city.cdcmonitor.utils.FileUtils;
import com.city.cdcmonitor.utils.ValidateUtils;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.*;

@Slf4j
@RestController
@RequestMapping("test")
public class TestController {
    @Value("${error.mail}")
    private String mail;
    @Resource
    private TokenFeign tokenFeign;
    @Resource
    private SendEmailService sendEmailService;
    @Resource
    private CheckHealthService checkHealthService;
    @Resource
    private CdcStatisticsService cdcStatisticsService;
    @Resource
    private JKCloudCommonMapper jkCloudCommonMapper;
    @Resource
    private MiddbCommonMapper middbCommonMapper;

    @RequestMapping("health")
    public String health() {
//        http://localhost:8080/test/health
        return "success";
    }

    @RequestMapping("testFeign")
    public String feign() {
//        http://localhost:8080/test/testFeign
        Map<String, Object> accessToken = tokenFeign.getAccessToken("6ea521e82b754743b029d6f906f67ffe",
                "5a1a68a474ff097ccb7d31591184a13a", null);
        return new Gson().toJson(accessToken);
    }

    @RequestMapping("testEmail")
    public String testEmail(HttpServletRequest request) {
//        http://localhost:8080/test/testEmail?text=
//        http://localhost:8080/test/testEmail
        String text = request.getParameter("text");
        if (ValidateUtils.isNotEmpty(text)) {
            sendEmailService.sendMail("cdc手动发送", text, false, mail.split(","));
        } else {
            String filePath = "C:\\citygis\\send-email";
            List<Map<String, Object>> files = new ArrayList<>();
            File file = new File(filePath);
            if (file.exists()) {
                String[] fileList = file.list();
                for (String fileName : fileList) {
                    Map<String, Object> map = new HashMap<>();
                    map.put("filePath", filePath + File.separator);
                    map.put("fileName", fileName);
                    files.add(map);
                }
                sendEmailService.sendMail("cdc手动发送附件", "", files, false, mail.split(","));
            }
        }
        return "success";
    }

    @RequestMapping("testJKCloud")
    public String testJKCloud(HttpServletRequest request) {
//        http://localhost:8080/test/testJKCloud?sql=
        String sql = request.getParameter("sql");
        List<LinkedHashMap<String, Object>> res = jkCloudCommonMapper.query(sql);
        return new Gson().toJson(res);
    }

    @RequestMapping("testMidDb")
    public String testMidDb(HttpServletRequest request) {
//        http://localhost:8080/test/testMidDb?sql=
        String sql = request.getParameter("sql");
        List<LinkedHashMap<String, Object>> res = middbCommonMapper.query(sql);
        return new Gson().toJson(res);
    }

    @RequestMapping("cdcStatistics")
    public String cdcStatistics(HttpServletRequest request) {
//        http://localhost:8080/test/cdcStatistics?day=2022-09-26
//        curl 127.0.0.1:8080/test/cdcStatistics?day=2022-09-27\&id=1000
        String id = request.getParameter("id");
        String day = request.getParameter("day");
        String start = request.getParameter("start");
        String end = request.getParameter("end");
        Integer idEquals = null;
        Integer idStart = null;
        Integer idEnd = null;
        if (ValidateUtils.isInteger(id)) {
            idEquals = Integer.parseInt(id);
        }
        if (ValidateUtils.isInteger(start)) {
            idStart = Integer.parseInt(start);
        }
        if (ValidateUtils.isInteger(end)) {
            idEnd = Integer.parseInt(end);
        }
        cdcStatisticsService.statistics(day, idEquals, null);
        return "success";
    }

    @RequestMapping("interfaceHealth")
    public String interfaceHealth(HttpServletRequest request) {
//        http://localhost:8080/test/interfaceHealth
        List<InterfaceHealthBean> healthBeans = checkHealthService.externalInterfaceHealth();
        String table = sendEmailService.getHtmlTable("接口监听", healthBeans);
        sendEmailService.sendMailAsTable("接口监听", table, false, mail.split(","));
        return new Gson().toJson(healthBeans);
    }

    @RequestMapping("checkDatabaseNameLock")
    public String checkDatabaseNameLock(HttpServletRequest request) {
//        http://localhost:8080/test/checkDatabaseNameLock
         cdcStatisticsService.checkDatabaseLock("middb");
         return "success";
    }

    @RequestMapping("getFilesLength")
    public String getFilesLength(HttpServletRequest request) {
        String filePath = request.getParameter("filePath");
        File file = new File(filePath);
        if (file.exists()) {
            String fileSpace = FileUtils.getFilesLengthDesc(FileUtils.getFilesLengthB(file));
            String usableSpace = FileUtils.getFilesLengthDesc(file.getUsableSpace());
            String fileTotalSpace = FileUtils.getFilesLengthDesc(file.getTotalSpace());
            return fileSpace + "/" + usableSpace + " (" + fileTotalSpace + ")";
        } else {
            return "0";
        }
    }
}
