package com.city.cdcmonitor.service;

import com.city.cdcmonitor.bean.ExportColumnBean;
import com.city.cdcmonitor.bean.FileSpaceBean;
import com.city.cdcmonitor.bean.InterfaceHealthBean;
import com.city.cdcmonitor.dao.jkcloud.mapper.JKCloudCommonMapper;
import com.city.cdcmonitor.dao.jkcloud.mapper.TabCdcStatisticsMapper;
import com.city.cdcmonitor.dao.jkcloud.model.TabCdcStatistics;
import com.city.cdcmonitor.dao.jkcloud.model.TabCdcStatisticsExample;
import com.city.cdcmonitor.dao.middb.mapper.MiddbCommonMapper;
import com.city.cdcmonitor.exception.CityException;
import com.city.cdcmonitor.utils.DateUtils;
import com.city.cdcmonitor.utils.EntityUtils;
import com.city.cdcmonitor.utils.ValidateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.text.StrSubstitutor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Service
public class CdcStatisticsService {
    @Value("${error.mail}")
    private String mailError;
    @Value("${send.statistics.mail}")
    private String mailSender;
    @Resource
    private SendEmailService sendEmailService;
    @Resource
    private CheckHealthService checkHealthService;
    @Resource
    private TabCdcStatisticsMapper tabCdcStatisticsMapper;
    @Resource
    private JKCloudCommonMapper jkCloudCommonMapper;
    @Resource
    private MiddbCommonMapper middbCommonMapper;

    public void statistics(String day, Integer idEquals, String taskId) {
        //邮件前缀
        String emailWarningPrefix = "";
        String emailWarningInfo = "";
        //查看数据库死锁情况, 单次查询只触发一次
        boolean checkMidDbLockFlag = true;
        boolean checkJkCloudLockFlag = true;
        //每周一执行判断
        /*boolean isMonday = false;
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        int weekDay = calendar.get(Calendar.DAY_OF_WEEK);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        if (weekDay == 2 && hour > 8 && hour < 18) {
            isMonday = true;
        }*/

        LinkedHashMap<String, List<LinkedHashMap<String, Object>>> resultMap = new LinkedHashMap<>();
        //sql查询
        TabCdcStatisticsExample example = new TabCdcStatisticsExample();
        if (idEquals != null) {
            example.or().andIdEqualTo(idEquals);
        } else if (ValidateUtils.isNotEmpty(taskId)) {
            example.or().andIdNotEqualTo(7).andTaskIdEqualTo(taskId);
        } else {
            example.or().andIdNotEqualTo(7);
        }
        /*} else {
            if (isMonday) {
                example.or().andIdNotEqualTo(7);
            } else {
                //10001以上为每周报表, 符合星期一的条件才执行
                example.or().andIdNotEqualTo(7).andTaskIdNotEqualTo("4").andTaskIdNotEqualTo("5");
            }
        }*/
        List<TabCdcStatistics> sqlList = tabCdcStatisticsMapper.selectByExample(example);
        for (TabCdcStatistics sqlTab : sqlList) {
            //统计查询语句
            List<LinkedHashMap<String, Object>> result = null;
            try {
                result = this.doSelect(sqlTab, day);
                if (sqlTab.getId() == 5) {
                    this.compare(result, day);
                }
                if (ValidateUtils.isNotEmpty(result)) {
                    resultMap.put(sqlTab.getEmailTitle(), result);
                }
            } catch (Exception e) {
                CityException.error(e);
                sendEmailService.sendMail("select error", "select error: <br/>" + CityException.getTrace(e), false, mailError.split(","));
                if (checkMidDbLockFlag && "middb".equals(sqlTab.getDatabaseName().trim().toLowerCase())) {
                    checkMidDbLockFlag = false;
                    //查看锁死情况
                    this.checkDatabaseLock(sqlTab.getDatabaseName());
                } else if (checkJkCloudLockFlag && "jkcloud".equals(sqlTab.getDatabaseName().trim().toLowerCase())) {
                    checkJkCloudLockFlag = false;
                    //查看锁死情况
                    this.checkDatabaseLock(sqlTab.getDatabaseName());
                }
            }
        }
        String table1 = "";
        if (ValidateUtils.isNotEmpty(resultMap)) {
            table1 = sendEmailService.getHtmlTable(resultMap);
        }

        //接口监听
        String table2 = "";
        if (ValidateUtils.isNotEmpty(taskId) && "1".equals(taskId)) {
            try {
                List<InterfaceHealthBean> healthBeans = checkHealthService.externalInterfaceHealth();
                if (ValidateUtils.isNotEmpty(healthBeans)) {
                    for (int i = 0; i < healthBeans.size(); i++) {
                        InterfaceHealthBean health = healthBeans.get(i);
                        if (health.getInterfaceName().contains("CommonLibraryForCDC")) {
                            if ("404".equals(health.getStatus())) {
                                emailWarningPrefix += "★";
                                emailWarningInfo += "(六统一接口已停止)";
                            }
                            break;
                        }
                    }
                    table2 = sendEmailService.getHtmlTable("六统一接口监听", healthBeans);
                }
            } catch (Exception e) {
                CityException.error(e);
                sendEmailService.sendMail("监听接口error", CityException.getTrace(e), false, mailError.split(","));
            }
        }

        //文件夹监听
        String table3 = "";
        if (ValidateUtils.isNotEmpty(taskId) && "1".equals(taskId)) {
            try {
                List<FileSpaceBean> fileLengthBeans = checkHealthService.fileLengthHealth();
                if (ValidateUtils.isNotEmpty(fileLengthBeans)) {
                    table3 = sendEmailService.getHtmlTable("六统一文件夹监听", fileLengthBeans);
                }
            } catch (Exception e) {
                CityException.error(e);
                sendEmailService.sendMail("获取磁盘空间error", CityException.getTrace(e), false, mailError.split(","));
            }
        }

        String taskName;
        if ("1".equals(taskId)) {
            taskName = "每小时";
        } else if ("2".equals(taskId)) {
            taskName = "每小时一刻";
        } else if ("3".equals(taskId)) {
            taskName = "每5分钟";
        } else if ("4".equals(taskId)) {
            taskName = "每周一9,15";
        } else if ("5".equals(taskId)) {
            taskName = "每周一12,18";
        } else {
            taskId = "";
            taskName = "手动";
        }

        String table = table1 + table2 + table3;
        if (ValidateUtils.isEmpty(table)) {
            return;
        }

        String emailTitle = emailWarningPrefix + "六统一监控报表" + taskId + "-" + taskName + "-" + DateUtils.dateToStringYYYYMMDDHHMMSS(new Date()) + emailWarningInfo;
        sendEmailService.sendMailAsTable(emailTitle, table, true, mailSender.split(","));
    }

    private List<LinkedHashMap<String, Object>> doSelect(TabCdcStatistics sqlTab, String day) {
        String databaseName = sqlTab.getDatabaseName();
        String sql = sqlTab.getSelectSql();
        List<LinkedHashMap<String, Object>> result = null;
        if (ValidateUtils.isNotEmpty(sql)) {
            //占位符处理
            if (sql.contains("${")) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                if (ValidateUtils.isEmpty(day)) {
                    day = DateUtils.dateToStringYYYYMMDD(new Date());
                }
                Map<String, String> map = new HashMap<>();
                map.put("now_date", "'" + day + "'");
                try {
                    Date yesterday = org.apache.commons.lang3.time.DateUtils.addDays(sdf.parse(day), -1);
                    map.put("yesterday_date", "'" + sdf.format(yesterday) + "'");
                } catch (ParseException e) {
                    e.printStackTrace();
                    CityException.error(e);
                }

                //上一周
                try {
                    map.put("last_mon", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.MONDAY));
                    map.put("last_tue", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.TUESDAY));
                    map.put("last_wed", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.WEDNESDAY));
                    map.put("last_thu", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.THURSDAY));
                    map.put("last_fri", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.FRIDAY));
                    map.put("last_sat", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.SATURDAY));
                    map.put("last_sun", DateUtils.getLastWeekDay(day, "yyyy-MM-dd", Calendar.SUNDAY));
                } catch (Exception e) {
                    e.printStackTrace();
                    CityException.error(e);
                }

                //替换占位符
                StrSubstitutor substitutor = new StrSubstitutor(map);
                sql = substitutor.replace(sql);
            }

            sqlTab.setSelectSql(sql);
            System.out.println(sql);
            if ("jkcloud".equals(databaseName.trim().toLowerCase())) {
                result = jkCloudCommonMapper.query(sql);
            } else if ("middb".equals(databaseName.trim().toLowerCase())) {
                result = middbCommonMapper.query(sql);
            } else {
                log.error("database not exists");
                sendEmailService.sendMail("database error", "database not exists: <br/>" + databaseName, false, mailError.split(","));
                return null;
            }
        }
        return result;
    }

    private void compare(List<LinkedHashMap<String, Object>> midDbList, String day) {
        TabCdcStatistics sqlTab = tabCdcStatisticsMapper.selectByPrimaryKey(7);
        List<LinkedHashMap<String, Object>> jKCloudList = this.doSelect(sqlTab, day);
        if (ValidateUtils.isNotEmpty(midDbList) && ValidateUtils.isNotEmpty(jKCloudList)) {
            Object operationTime = midDbList.get(0).get("操作时间");
            //第一条记录为表格表头, 把‘插入_更新_删除总量’放到最后
            for (int i = 0; i < midDbList.size(); i++) {
                LinkedHashMap<String, Object> midDbData = midDbList.get(i);
                //重新排序, 插入_更新_删除总量放到最后
                midDbData.put("插入_更新_删除总量", midDbData.get("插入_更新_删除总量"));
                String tableName = (String) midDbData.get("操作表");
                Object num = midDbData.get("插入_更新_删除总量");
                int count = Integer.parseInt(num.toString());
                for (int j = 0; j < jKCloudList.size(); j++) {
                    LinkedHashMap<String, Object> jKCloudData = jKCloudList.get(j);
                    String tableNameJk = (String) jKCloudData.get("表名");
                    Object numJk = jKCloudData.get("表当前实际数量");
                    int countJk = Integer.parseInt(numJk.toString());
                    if (tableName.trim().equals(tableNameJk.trim())) {
                        midDbData.put("jkcloud实际入库数量", numJk);
                        midDbData.put("数量对比差值", String.valueOf(count - countJk));
                        jKCloudList.remove(j);
                        break;
                    } else {
                        LinkedHashMap<String, Object> data = new LinkedHashMap<>();
                        data.put("日志_Select数量", "0");
                        data.put("日志_Insert数量", "0");
                        data.put("表当日实际Insert数量", "0");
                        data.put("日志_Update数量", "0");
                        data.put("表当日实际update数量", "0");
                        data.put("日志_Delete数量", "0");
                        data.put("表当日实际Delete数量", "0");
                        data.put("插入失败", "0");
                        data.put("更新失败", "0");
                        data.put("删除失败", "0");
                        data.put("操作时间", operationTime);
                        data.put("操作表", jKCloudData.get("表名"));
                        data.put("inx", jKCloudData.get("idx"));
                        data.put("插入_更新_删除总量", "0");
                        data.put("jkcloud实际入库数量", numJk);
                        data.put("数量对比差值", String.valueOf(0 - countJk));
                        midDbList.add(i, data);
                        jKCloudList.remove(j);
                        j--;
                        i++;
                    }
                }
            }

            //将jKCloud多余记录插入midDb
            for (LinkedHashMap<String, Object> jKCloudData : jKCloudList) {
                Object numJk = jKCloudData.get("表当前实际数量");
                int countJk = Integer.parseInt(numJk.toString());
                LinkedHashMap<String, Object> data = new LinkedHashMap<>();
                data.put("日志_Select数量", "0");
                data.put("日志_Insert数量", "0");
                data.put("表当日实际Insert数量", "0");
                data.put("日志_Update数量", "0");
                data.put("表当日实际update数量", "0");
                data.put("日志_Delete数量", "0");
                data.put("表当日实际Delete数量", "0");
                data.put("插入失败", "0");
                data.put("更新失败", "0");
                data.put("删除失败", "0");
                data.put("操作时间", operationTime);
                data.put("操作表", jKCloudData.get("表名"));
                data.put("inx", jKCloudData.get("idx"));
                data.put("插入_更新_删除总量", "0");
                data.put("jkcloud实际入库数量", numJk);
                data.put("数量对比差值", String.valueOf(0 - countJk));
                midDbList.add(data);
            }

            try {
                Calendar calendar = Calendar.getInstance();
                int h = calendar.get(Calendar.HOUR_OF_DAY);
                if (h > 7) {
                    doInsert(midDbList);
                }
            } catch (Exception e) {
                sendEmailService.sendMail("insert error", "insert error: <br/>" + CityException.getTrace(e), false, mailError.split(","));
            }
        }
    }

    private void doInsert(List<LinkedHashMap<String, Object>> result) {
        //判断是否已有记录
        Object operationTime = result.get(0).get("操作时间");
        List<LinkedHashMap<String, Object>> query = middbCommonMapper.query("select count(1) as count From [MIDDB].[dbo].[view_昨日日报] where [操作时间] = '" + operationTime + "'");
        Object count = query.get(0).get("count");
        int num = Integer.parseInt(count.toString());
        if (num > 0) {
            return;
        }

        for (LinkedHashMap<String, Object> data : result) {
            StringBuilder columns = new StringBuilder();
            StringBuilder values = new StringBuilder();
            List<ExportColumnBean> columnList = EntityUtils.getExportColumnList5();
            for (ExportColumnBean exportColumnBean : columnList) {
                if (columns.length() > 0) {
                    columns.append(",");
                }
                columns.append("[").append(exportColumnBean.getInsertColumnName()).append("]");
                if (values.length() > 0) {
                    values.append(",");
                }
                Object v = data.get(exportColumnBean.getColumnName());
                if (v == null || "null".equals(v)) {
                    v = "0";
                }
                values.append("'").append(v.toString()).append("'");
            }
            String sql = "insert into [MIDDB].[dbo].[view_昨日日报] " + "(" + columns + ") values (" + values + ")";
            log.info("insert sql = \n" + sql);
            middbCommonMapper.insert(sql);
        }
    }

    public void checkDatabaseLock(String databaseName) {
        StringBuilder errorMsg = new StringBuilder("error: ");
        String showTablesSql = "select name from sys.tables";
        List<LinkedHashMap<String, Object>> tables = null;
        try {
            if ("middb".equals(databaseName.trim().toLowerCase())) {
                tables = middbCommonMapper.query(showTablesSql);
            } else {
                tables = jkCloudCommonMapper.query(showTablesSql);
            }
            for (int i = 0; i < tables.size(); i++) {
                Object tableName = tables.get(i).get("name");
                String sql = "select top 1 * from [" + tableName.toString() + "]";
                try {
                    if ("middb".equals(databaseName.trim().toLowerCase())) {
                        middbCommonMapper.query2(sql);
                    } else {
                        jkCloudCommonMapper.query2(sql);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    errorMsg.append("<br/>").append(tableName.toString());
                }
            }
        } catch (Exception e) {
            errorMsg.append(showTablesSql).append("<br/>");
            errorMsg.append(CityException.getTrace(e));
        }
        System.out.println("errorMsg:\n" + errorMsg.toString());
        sendEmailService.sendMail(databaseName + "死锁情况", errorMsg.toString(), false, mailError.split(","));
    }
}
