package com.city.cdcmonitor.service;

import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.*;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.stereotype.Service;

import java.util.Properties;

/**
 * 心跳检测
 */
@Slf4j
@Service
public class KafkaHeartbeatService {

    public void heartbeat(String bootstrapServers, int timeout) {
        Properties props = new Properties();
        // 重试次数，0为不启用重试机制
        props.put(ProducerConfig.RETRIES_CONFIG, 3);
        // 生产者空间不足时，send()被阻塞的时间，默认60s
        props.put(ProducerConfig.MAX_BLOCK_MS_CONFIG, timeout);
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        props.put(ProducerConfig.ACKS_CONFIG, "1");
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        // 消费者默认支持解压，所以压缩设置在生产者，消费者无需设置。
        props.put(ProducerConfig.COMPRESSION_TYPE_CONFIG, "none");

        KafkaProducer<String, String> producer = null;

        try {
            producer = new KafkaProducer<String, String>(props);
            ProducerRecord<String, String> record = new ProducerRecord<>(
                    "quickstart-events",
                    "test-value-" + System.currentTimeMillis());

            producer.send(record, (metadata, e) -> {
                if (e != null) {
                    // 消息发送失败
                    throw new RuntimeException(e);
                }
            });
        } finally {
            if (producer != null) {
                producer.close();
            }
        }
    }
}
