package com.city.cdcmonitor.task;

import com.city.cdcmonitor.service.CdcStatisticsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 统计
 */
@Slf4j
@Component
@ConditionalOnProperty(prefix = "task", name = "monitor", havingValue = "true")
public class Statistics {
    @Resource
    private CdcStatisticsService cdcStatisticsService;

    /**
     * 每小时一次, 整点
     */
    @Async
    @Scheduled(cron = "0 0 0/1 * * ?")
    public void hourlyTask1() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [1号任务-每小时一次统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "1");
        } catch (Exception e) {
            log.error(" ### [1号任务-每小时一次统计任务],执行异常： ", e);
        }
        log.info(" ### [1号任务-每小时一次统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }

    /**
     * 每小时一次, 2号方法, 整点15分
     */
    @Async
    @Scheduled(cron = "0 15 0/1 * * ?")
    public void hourlyTask2() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [2号任务-每小时一次统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "2");
        } catch (Exception e) {
            log.error(" ### [3号任务-每小时一次统计任务],执行异常： ", e);
        }
        log.info(" ### [3号任务-每小时一次统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }

    /**
     * 每5分钟一次
     */
    @Async
    @Scheduled(cron = "0 0/5 * * * ?")
    public void minutelyTask3() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [3号任务-每5分钟一次统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "3");
        } catch (Exception e) {
            log.error(" ### [3号任务-每小时一次统计任务],执行异常： ", e);
        }
        log.info(" ### [3号任务-每小时一次统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }

    /**
     * 每周一次, 星期一9点15点
     */
    @Async
    @Scheduled(cron = "0 30 9,15 ? * MON")
    public void weeklyTask4() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [4号任务-每周统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "4");
        } catch (Exception e) {
            log.error(" ### [4号任务-每周统计任务],执行异常： ", e);
        }
        log.info(" ### [4号任务-每周统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }

    /**
     * 每周一次, 星期一12点18点
     */
    @Async
    @Scheduled(cron = "0 30 12,18 ? * MON")
    public void weeklyTask5() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [5号任务-每周统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "5");
        } catch (Exception e) {
            log.error(" ### [5号任务-每周统计任务],执行异常： ", e);
        }
        log.info(" ### [5号任务-每周统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }

    /**
     * 每月一次, 一号8点
     */
    @Async
    @Scheduled(cron = "0 0 8 1 * ?")
    public void weeklyTask6() {
        long startTime = System.currentTimeMillis();
        log.info(" ### [6号任务-每月统计任务], start...");
        try {
            cdcStatisticsService.statistics(null, null, "6");
        } catch (Exception e) {
            log.error(" ### [6号任务-每月统计任务],执行异常： ", e);
        }
        log.info(" ### [6号任务-每月统计任务] end, 执行耗时：" + (System.currentTimeMillis() - startTime) + "ms");
    }
}
