package com.city.cdcmonitor.utils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.EnvironmentAware;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

@Slf4j
@Component
public class PropertyConfig implements EnvironmentAware {
    private static Environment env;
    private static String profile;
    private static String propertyPath;
    private static final Map<String, Properties> propMap = new HashMap<>();

    @Override
    public void setEnvironment(Environment environment) {
        env = environment;
    }

    @Value("${spring.profiles.active}")
    public void setProfile(String active) {
        profile = active;
    }

    @Value("${city.properties.path}")
    public void setPropertyPath(String path) {
        if ("dev".equals(path.trim())) {
            path = System.getProperty("user.dir") + File.separator + "src" + File.separator + "main" + File.separator + "resources";
        }
        log.info("###路径: " + path);
        propertyPath = path;
    }

    public static String getConfig(String key) {
        return getConfig(null, key);
    }

    public static String getConfig(String prop, String key) {
        if (ValidateUtils.isEmpty(prop)) {
            prop = "city";
        }
        loadProp(prop);
        Properties props = propMap.get(prop);
        if (!props.containsKey(key)) {
            log.error("Prop [" + prop + "]no this key:" + key);
            throw new RuntimeException("Prop [" + prop + "]no this key:" + key);
        }
        return props.getProperty(key).trim();
    }

    public static Map<String, String> getExternalInterfaceHealthIp() {
        Map<String, String> ips = new HashMap<>();
        loadProp("external-interface-health");
        Properties props = propMap.get("external-interface-health");
        for (Object key : props.keySet()) {
            ips.put(key.toString(), props.get(key).toString());
        }
        return ips;
    }

    public static String getPropertiesValue(String key) {
        return env.getProperty(key);
    }

    public static void loadProp(String propName) {
        Properties properties = new Properties();
        FileInputStream fis = null;
        InputStreamReader isReader = null;
        BufferedReader bReader = null;
        try {
            fis = new FileInputStream(propertyPath + File.separator + propName + ".properties");
            isReader = new InputStreamReader(fis, StandardCharsets.UTF_8);
            bReader = new BufferedReader(isReader);
            properties.load(bReader);
        } catch (Exception e) {
            log.error("read properties error" + e);
            throw new RuntimeException("read properties error");
        } finally {
            try {
                if (bReader != null) {
                    bReader.close();
                }
            } catch (IOException e) {
                log.error("close BufferedReader error", e);
            }
            try {
                if (isReader != null) {
                    isReader.close();
                }
            } catch (IOException e) {
                log.error("close InputStreamReader error", e);
            }
            try {
                if (fis != null)
                    fis.close();
            } catch (IOException e) {
                log.error("close FileInputStream error", e);
            }
        }
        propMap.put(propName, properties);
    }
}
